si <- function(x, genes, group, members, median.gene=FALSE,median.probeset=FALSE,unlogged=TRUE) {

  if(missing(group)) {
   tr1 <- members[[1]]
   tr2 <- members[[2]]
  }
  else {
    tr1 <- group.indices(x, group,members[1]) 
    tr2 <- group.indices(x, group,members[2]) 
  }

  table <- .g.to.e.p.exp(x, genes)

  #Can return from here - if nothing found...
  if(is.null(table)) {
    r <- matrix(nrow=0,ncol=3)
    colnames(r) <- c("si","p.score","t.statistic")
    return(r)
  }

  gene.list <- split(table, table$gene)

  lapply(gene.list,function(a) {
    a   <- a[!duplicated(a$"probeset_name"),,drop=FALSE]
 
    idx <- c(tr1,tr2)
    tmp <- as.matrix(a[,idx,drop=FALSE])

    tr1 <- seq_along(tr1)
    tr2 <- length(tr1) + seq_along(tr2)
    idx <- c(tr1,tr2)
 
    if(median.gene) { avfun.g <- median }
    else            { avfun.g <- mean   }
    if(median.probeset) { avfun.p <- median }
    else                { avfun.p <- mean   }

    
    if(unlogged) {
      tmp <- 2^tmp
      avs <- apply(tmp,2,avfun.g)
      tmp <- sweep(tmp,2,avs,"/")
      s1  <- apply(tmp[,tr1,drop=FALSE],1,avfun.p)
      s2  <- apply(tmp[,tr2,drop=FALSE],1,avfun.p)
      si  <- log2(s1/s2)
    }
    else {
      avs <- apply(tmp,2,avfun.g)
      tmp <- sweep(tmp,2,avs,"-")
      s1  <- apply(tmp[,tr1,drop=FALSE],1,avfun.p)
      s2  <- apply(tmp[,tr2,drop=FALSE],1,avfun.p)
      si <- s1-s2
 
    }

    fac <- as.factor(idx %in% tr2)
 
    r <- rowttests(tmp,fac)
    r <- cbind(si,r[,c(3,1),drop=FALSE])

    rownames(r) <- a$"probeset_name"
    colnames(r) <- c("si","p.score","t.statistic")
    r
  })
}


.splicing.index.affy <- function(x, gene,group,members, median.gene=TRUE)
{ 
  tr1 <- group.indices(x, group,members[1]) 
  tr2 <- group.indices(x, group,members[2]) 
  tr <- c(tr1, tr2)
  si <- NULL
  table <- gene.to.exon.probeset.expr(x, gene)
  table <- .unique.gene.exon(table)
  if (!is.vector(table) & !is.null(table)) 
    {
    gle <- vector(length=length(tr1) + length(tr2))  # 
    gle[1:(length(gle))] <- 0
    #unlog'em'all
    table[,tr] <- 2^(table[,tr])
    if (median.gene)
    { 
      for (t in tr) 
      {
      gle[t] <- median(as.matrix(table[,t]))
      }
    }
    else
    {
      for (t in tr) 
      {
      gle[t] <- mean(as.matrix(table[,t]))
      }
    }
    for (t in tr) 
    {
     table[,t]=table[,t]/gle[t]
    }
    si <- NULL
    for (p in rownames(table)) 
    {
    si <- c(si,log2(mean(as.numeric(table[p,tr1]))/mean(as.numeric(table[p,tr2]))))
    }
    fac <- pData(x)[,group]
    pvals <-  rowttests(as.matrix(table[,tr]),fac)$p.value

     names(si) <- table[, "probeset_name"]
     names(pvals) <- table[, "probeset_name"]
     out <- NULL
     out$si <- si
     out$pvals <- pvals
     si <- out
    }
  si
}

.splicing.index <- function(x, gene,group,members, median.gene=TRUE, vector.out=FALSE)
{ 
  tr1 <- group.indices(x, group,members[1]) 
  tr2 <- group.indices(x, group,members[2]) 
  si <- NULL
  table <- gene.to.exon.probeset.expr(x, gene)
  table <- .unique.gene.exon(table)
  if (!is.vector(table) & !is.null(table)) 
    {
    if (median.gene)
    {
      gle1 <- log2(median(2^(as.matrix(table[,tr1]))))
      gle2 <- log2(median(2^(as.matrix(table[,tr2]))))
    }
    else
    {
      gle1 <- log2(mean(2^(as.matrix(table[,tr1]))))
      gle2 <- log2(mean(2^(as.matrix(table[,tr2]))))
    }
    corfact <- gle1 - gle2
    if (length(tr1)>1) indexp1 <- log2(apply(2^table[,tr1], 1, mean))
    else indexp1 <- table[,tr1]
    if (length(tr2)>1) indexp2 <- log2(apply(2^table[,tr2], 1, mean))
    else indexp2 <- table[,tr2]
    si <-  indexp1 -indexp2 - corfact
    if (!vector.out) si <- cbind(si, table)
    else names(si) <- table[, "probeset_name"]
    }
  si
}

splicing.index <- function(x, genes, group, members, median.gene=TRUE, vector.out=TRUE, by.chip=FALSE)
{
  .Deprecated(new="si",msg="This function is deprecated and will dissapear in future version of exonmap. Please use the alternate function 'si', which replaces it.")
  
  out <- NULL
  if (!by.chip) {
    for (i in seq_along(genes)) { 
      tmp.table <- .splicing.index (x, genes[i],group,members, median.gene=median.gene, vector.out=vector.out)
      if (!vector.out & !is.null(tmp.table)) out <- rbind(out, tmp.table)
      if (vector.out & !is.null(tmp.table)) out <- c(out, tmp.table)
    }
  }

  if (by.chip) {
    stop("Not implemented. See the function 'si', which provides this functionality.")
  }

  out
}

.unique.gene.exon <- function(table)
{
  out <- NULL
  if (class(table)=="data.frame")
  {
    for (p in unique(table[,"probeset_name"]))
    {
       i <- which(table[,"probeset_name"]==p)[1]
       out <- rbind(out, table[i,])
    }
  }
  else out <- table
  out
}
